@@ -46,6 +46,7 @@ struct SimpleScanPrivate
     GtkTreeModel *mode_model;
     GtkWidget *preview_box, *preview_area, *preview_scroll;
     GtkWidget *page_delete_menuitem, *crop_rotate_menuitem;
+    GtkWidget *stop_menuitem, *stop_toolbutton;
 
     GtkWidget *authorize_dialog;
     GtkWidget *authorize_label;
@@ -54,6 +55,7 @@ struct SimpleScanPrivate
     GtkWidget *preferences_dialog;
     GtkWidget *device_combo, *text_dpi_combo, *photo_dpi_combo;
     GtkTreeModel *device_model, *text_dpi_model, *photo_dpi_model;
+    gboolean setting_devices, user_selected_device;
 
     Book *book;
     BookView *book_view;
@@ -137,13 +139,82 @@ ui_authorize (SimpleScan *ui, const gchar *resource, gchar **username, gchar **p
     *password = g_strdup (gtk_entry_get_text (GTK_ENTRY (ui->priv->password_entry)));
 }
 
+
+void device_combo_changed_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+device_combo_changed_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (ui->priv->setting_devices)
+        return;
+    ui->priv->user_selected_device = TRUE;
+}
+
+
 void
 ui_set_scan_devices (SimpleScan *ui, GList *devices)
 {
+    GList *d;
+    gboolean have_selection = FALSE;
+    gint index;
     GtkTreeIter iter;
-    GList *i;
-    gboolean have_iter;
-    gboolean have_selection;
+  
+    ui->priv->setting_devices = TRUE;
+ 
+    /* If the user hasn't chosen a scanner choose the best available one */
+    if (ui->priv->user_selected_device)
+        have_selection = gtk_combo_box_get_active (GTK_COMBO_BOX (ui->priv->device_combo)) >= 0;
+
+    /* Add new devices */
+    index = 0;
+    for (d = devices; d; d = d->next) {
+        ScanDevice *device = (ScanDevice *) d->data;
+        gint n_delete = -1;
+
+        /* Find if already exists */
+        if (gtk_tree_model_iter_nth_child (ui->priv->device_model, &iter, NULL, index)) {
+            gint i = 0;
+            do {
+                gchar *name;
+                gboolean matched;
+
+                gtk_tree_model_get (ui->priv->device_model, &iter, 0, &name, -1);
+                matched = strcmp (name, device->name) == 0;
+                g_free (name);
+
+                if (matched) {
+                    n_delete = i;
+                    break;
+                }
+                i++;
+            } while (gtk_tree_model_iter_next (ui->priv->device_model, &iter));
+        }
+      
+        /* If exists, remove elements up to this one */
+        if (n_delete >= 0) {
+            gint i;
+
+            for (i = 0; i < n_delete; i++) {
+                gtk_tree_model_iter_nth_child (ui->priv->device_model, &iter, NULL, index);
+                gtk_list_store_remove (GTK_LIST_STORE (ui->priv->device_model), &iter);
+            }
+        }
+        else {
+            gtk_list_store_insert (GTK_LIST_STORE (ui->priv->device_model), &iter, index);
+            gtk_list_store_set (GTK_LIST_STORE (ui->priv->device_model), &iter, 0, device->name, 1, device->label, -1);
+        }
+        index++;
+    }
+
+    /* Remove any remaining devices */
+    while (gtk_tree_model_iter_nth_child (ui->priv->device_model, &iter, NULL, index))
+        gtk_list_store_remove (GTK_LIST_STORE (ui->priv->device_model), &iter);
+
+    /* Select the first available device */
+    if (!have_selection && devices != NULL)
+        gtk_combo_box_set_active (GTK_COMBO_BOX (ui->priv->device_combo), 0);
+
+    ui->priv->setting_devices = FALSE;
 
     if (!ui->priv->have_device_list) {
         ui->priv->have_device_list = TRUE;
@@ -157,46 +228,6 @@ ui_set_scan_devices (SimpleScan *ui, GList *devices)
                            FALSE);
         }
     }
-  
-    have_selection = gtk_combo_box_get_active (GTK_COMBO_BOX (ui->priv->device_combo)) >= 0;
-  
-    /* Remove disappeared devices */
-
-    do {
-        for (have_iter = gtk_tree_model_get_iter_first (ui->priv->device_model, &iter);
-             have_iter;
-             have_iter = gtk_tree_model_iter_next (ui->priv->device_model, &iter)) {
-            gchar *name;
-
-            gtk_tree_model_get (ui->priv->device_model, &iter, 0, &name, -1);
-            for (i = devices; i; i = i->next)
-                if (strcmp (name, ((ScanDevice *) i->data)->name) == 0)
-                    break;
-            g_free (name);
-
-            /* Device was removed */
-            if (i == NULL) {
-                gtk_list_store_remove (GTK_LIST_STORE (ui->priv->device_model), &iter);
-                break;
-            }
-        }
-    } while (have_iter);
-
-    /* Add new devices */
-    for (i = devices; i; i = i->next) {
-        ScanDevice *device = (ScanDevice *) i->data;
-
-        if (!find_scan_device (ui, device->name, &iter)) {
-            gtk_list_store_append (GTK_LIST_STORE (ui->priv->device_model), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (ui->priv->device_model), &iter, 0, device->name, 1, device->label, -1);
-
-            /* Select this device if none selected */
-            if (!have_selection) {
-                gtk_combo_box_set_active_iter (GTK_COMBO_BOX (ui->priv->device_combo), &iter);
-                have_selection = TRUE;
-            }
-        }
-    }
 }
 
 
@@ -233,13 +264,16 @@ ui_set_selected_device (SimpleScan *ui, const gchar *device)
 static void
 add_default_page (SimpleScan *ui)
 {
+    Page *page;
+  
     if (book_get_n_pages (ui->priv->book) > 0)
         return;
 
-    book_append_page (ui->priv->book,
-                      ui->priv->default_page_width, ui->priv->default_page_height,
-                      ui->priv->default_page_dpi,
-                      ui->priv->default_page_orientation);
+    page = book_append_page (ui->priv->book,
+                             ui->priv->default_page_width, ui->priv->default_page_height,
+                             ui->priv->default_page_dpi,
+                             ui->priv->default_page_orientation);
+    book_view_select_page (ui->priv->book_view, page);
 }
 
 
@@ -407,7 +441,7 @@ page_selected_cb (BookView *view, Page *page, SimpleScan *ui)
     
     if (page_has_crop (page)) {
         char *crop_name;
-
+      
         // FIXME: Make more generic
         crop_name = page_get_named_crop (page);
         if (crop_name) {
@@ -432,7 +466,7 @@ page_selected_cb (BookView *view, Page *page, SimpleScan *ui)
         name = "no_crop_menuitem";
 
     gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (gtk_builder_get_object (ui->priv->builder, name)), TRUE);
-    
+
     ui->priv->updating_page_menu = FALSE;
 }
 
@@ -462,7 +496,6 @@ rotate_right_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
         return;
     page = book_view_get_selected (ui->priv->book_view);
     page_rotate_right (page);
-    ui->priv->default_page_orientation = page_get_orientation (page);
 }
 
 
@@ -657,6 +690,7 @@ save_file_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
     GtkTreeIter iter;
     GtkTreeViewColumn *column;
     const gchar *extension;
+    gchar *directory;
     gint i;
 
     struct
@@ -673,6 +707,13 @@ save_file_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
         { NULL, NULL }
     };
 
+    /* Get directory to save to */
+    directory = gconf_client_get_string (ui->priv->client, "/apps/simple-scan/save_directory", NULL);
+    if (!directory || directory[0] == '\0') {
+        g_free (directory);
+        directory = g_strdup (g_get_user_special_dir (G_USER_DIRECTORY_DOCUMENTS));
+    }
+
     dialog = gtk_file_chooser_dialog_new (/* Save dialog: Dialog title */
                                           _("Save As..."),
                                           GTK_WINDOW (ui->priv->window),
@@ -682,7 +723,9 @@ save_file_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
                                           NULL);
     gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
     gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog), FALSE);
+    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), directory);
     gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), ui->priv->default_file_name);
+    g_free (directory);
 
     /* Filter to only show images by default */
     filter = gtk_file_filter_new ();
@@ -690,6 +733,7 @@ save_file_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
                               /* Save dialog: Filter name to show only image files */
                               _("Image Files"));
     gtk_file_filter_add_pixbuf_formats (filter);
+    gtk_file_filter_add_mime_type (filter, "application/pdf");
     gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
     filter = gtk_file_filter_new ();
     gtk_file_filter_set_name (filter,
@@ -747,6 +791,11 @@ save_file_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 
         g_free (uri);
     }
+
+    gconf_client_set_string (ui->priv->client, "/apps/simple-scan/save_directory",
+                             gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dialog)),
+                             NULL);
+  
     gtk_widget_destroy (dialog);
 }
 
@@ -791,7 +840,10 @@ G_MODULE_EXPORT
 void
 email_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 {
-    g_signal_emit (G_OBJECT (ui), signals[EMAIL], 0);
+    gchar *mode;
+    mode = get_document_hint (ui);
+    g_signal_emit (G_OBJECT (ui), signals[EMAIL], 0, mode);
+    g_free (mode);
 }
 
 
@@ -966,11 +1018,17 @@ window_delete_event_cb (GtkWidget *widget, GdkEvent *event, SimpleScan *ui)
 
 
 static void
-page_changed_cb (Page *page, SimpleScan *ui)
+page_size_changed_cb (Page *page, SimpleScan *ui)
 {
     ui->priv->default_page_width = page_get_scan_width (page);
     ui->priv->default_page_height = page_get_scan_height (page);
     ui->priv->default_page_dpi = page_get_dpi (page);
+}
+
+
+static void
+page_orientation_changed_cb (Page *page, SimpleScan *ui)
+{
     ui->priv->default_page_orientation = page_get_orientation (page);
 }
 
@@ -982,7 +1040,8 @@ page_added_cb (Book *book, Page *page, SimpleScan *ui)
     ui->priv->default_page_height = page_get_scan_height (page);
     ui->priv->default_page_dpi = page_get_dpi (page);
     ui->priv->default_page_orientation = page_get_orientation (page);
-    g_signal_connect (page, "image-changed", G_CALLBACK (page_changed_cb), ui);
+    g_signal_connect (page, "size-changed", G_CALLBACK (page_size_changed_cb), ui);
+    g_signal_connect (page, "orientation-changed", G_CALLBACK (page_orientation_changed_cb), ui);
 }
 
 
@@ -1079,6 +1138,8 @@ ui_load (SimpleScan *ui)
     ui->priv->preview_scroll = GTK_WIDGET (gtk_builder_get_object (builder, "preview_scrollbar"));
     ui->priv->page_delete_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "page_delete_menuitem"));
     ui->priv->crop_rotate_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "crop_rotate_menuitem"));
+    ui->priv->stop_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "stop_scan_menuitem"));
+    ui->priv->stop_toolbutton = GTK_WIDGET (gtk_builder_get_object (builder, "stop_toolbutton"));
 
     renderer = gtk_cell_renderer_text_new();
     gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->mode_combo), renderer, TRUE);
@@ -1205,13 +1266,8 @@ ui_set_scanning (SimpleScan *ui, gboolean scanning)
 {
     ui->priv->scanning = scanning;
     gtk_widget_set_sensitive (ui->priv->page_delete_menuitem, !scanning);
-}
-
-
-void
-ui_set_have_scan (SimpleScan *ui, gboolean have_scan)
-{
-    //FIXME: gtk_widget_set_sensitive (ui->priv->actions_box, have_scan);
+    gtk_widget_set_sensitive (ui->priv->stop_menuitem, scanning);
+    gtk_widget_set_sensitive (ui->priv->stop_toolbutton, scanning);
 }
 
 
@@ -1325,8 +1381,8 @@ ui_class_init (SimpleScanClass *klass)
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (SimpleScanClass, email),
                       NULL, NULL,
-                      g_cclosure_marshal_VOID__VOID,
-                      G_TYPE_NONE, 0);
+                      g_cclosure_marshal_VOID__STRING,
+                      G_TYPE_NONE, 1, G_TYPE_STRING);
     signals[QUIT] =
         g_signal_new ("quit",
                       G_TYPE_FROM_CLASS (klass),
